@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 """
     Game Conqueror: a graphical game cheating tool, using scanmem as its backend
     
@@ -103,7 +103,6 @@ TYPESIZES = {'int8':1
 
 class GameConqueror():
     def __init__(self):
-        Gtk.Settings.get_default().set_long_property('gtk-tooltip-timeout', 0, '')
         ##################################
         # init GUI
         self.builder = Gtk.Builder()
@@ -358,10 +357,10 @@ class GameConqueror():
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
             try:
-                with open(dialog.get_filename(), 'rb') as f:
+                with open(dialog.get_filename(), 'r') as f:
                     obj = json.load(f)
                     for row in obj['cheat_list']:
-                        self.add_to_cheat_list(row[3],row[5],row[4],row[2])
+                        self.add_to_cheat_list(row[3],row[5],row[4],row[2],True)
             except:
                 pass
         dialog.destroy()
@@ -378,7 +377,7 @@ class GameConqueror():
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
             try:
-                with open(dialog.get_filename(), 'wb') as f:
+                with open(dialog.get_filename(), 'w') as f:
                     obj = {'cheat_list' : [list(i) for i in self.cheatlist_liststore]}
                     json.dump(obj, f);
             except:
@@ -566,20 +565,38 @@ class GameConqueror():
             return True
         return False
 
-    def cheatlist_toggle_lock_cb(self, cellrenderertoggle, path, data=None):
+    def cheatlist_toggle_lock(self, row):
+        if self.cheatlist_liststore[row][6]: # valid
+            locked = self.cheatlist_liststore[row][1]
+            locked = not locked
+            self.cheatlist_liststore[row][1] = locked
+        if locked:
+            #TODO: check value(valid number & not overflow), if failed, unlock it and do nothing
+            pass
+        else:
+            #TODO: update its value?
+            pass
+        return True
+
+    def cheatlist_toggle_lock_cb(self, cellrenderertoggle, row_str, data=None):
         pathlist = self.cheatlist_tv.get_selection().get_selected_rows()[1]
+        if not row_str:
+            return True
+        cur_row = int(row_str)
+        # check if the current row is part of the selection
+        found = False
         for path in pathlist:
             row = path[0]
-            if self.cheatlist_liststore[row][6]: # valid
-                locked = self.cheatlist_liststore[row][1]
-                locked = not locked
-                self.cheatlist_liststore[row][1] = locked
-            if locked:
-                #TODO: check value(valid number & not overflow), if failed, unlock it and do nothing
-                pass
-            else:
-                #TODO: update its value?
-                pass
+            if row == cur_row:
+                found = True
+                break
+        if not found:
+            self.cheatlist_toggle_lock(cur_row)
+            return True
+        # the current row is part of the selection
+        for path in pathlist:
+            row = path[0]
+            self.cheatlist_toggle_lock(row)
         return True
 
     def cheatlist_toggle_lock_flag_cb(self, cell, path, new_text, data=None):
@@ -683,10 +700,10 @@ class GameConqueror():
         if typename in TYPENAMES_G2STRUCT:
             return struct.unpack(TYPENAMES_G2STRUCT[typename], databytes)[0]
         elif typename == 'string':
-            databytes = str(databytes.decode())
+            databytes = str(u(databytes))
             return repr('%s'%(databytes,))[1:-1]
         elif typename == 'bytearray':
-            databytes = databytes.decode()
+            databytes = u(databytes)
             return ' '.join(['%02x'%ord(i) for i in databytes])
         else:
             return databytes
@@ -760,7 +777,7 @@ class GameConqueror():
         Gdk.threads_leave()
         return True
 
-    def add_to_cheat_list(self, addr, value, typestr, description=_('No Description')):
+    def add_to_cheat_list(self, addr, value, typestr, description=_('No Description'), at_end=False):
         # determine longest possible type
         types = typestr.split()
         vt = typestr
@@ -768,7 +785,10 @@ class GameConqueror():
             if t in TYPENAMES_S2G:
                 vt = TYPENAMES_S2G[t]
                 break
-        self.cheatlist_liststore.prepend(['=', False, description, addr, vt, str(value), True])
+        if at_end:
+            self.cheatlist_liststore.append(['=', False, description, addr, vt, str(value), True])
+        else:
+            self.cheatlist_liststore.prepend(['=', False, description, addr, vt, str(value), True])
 
     def get_process_list(self):
         return [list(map(str.strip, e.strip().split(' ',2))) for e in os.popen('ps -wweo pid=,user=,command= --sort=-pid').readlines()]
@@ -911,11 +931,11 @@ class GameConqueror():
             # temporarily disable model for scanresult_liststore for the sake of performance
             self.scanresult_liststore.clear()
             for line in lines:
-                line = str(line.decode())
+                line = str(u(line))
                 line = line[line.find(']')+1:]
                 (a, o, rt, v, t) = list(map(str.strip, line.split(',')[:5]))
                 a = '%x'%(int(a,16),)
-                o = '%x'%(int(o[5:],16),)
+                o = '%x'%(int(o.split('+')[1],16),)
                 t = t[1:-1]
                 if t == 'unknown':
                     continue
